unit DiskTools;

interface

uses SysUtils, Windows, Classes;

function GetPartitionType(PartitionType : Byte; Default : String) : String;
function FormatByteValue(bytes : Int64) : String;

function GetVolumesOnPhysicalDisk(PhysicalDiskName : String; VolumeNames : TStrings) : Boolean; overload;
function GetVolumesOnPhysicalDisk(PhysicalDiskNumber : Integer; VolumeNames : TStrings) : Boolean; overload;
function GetVolumeMountPoints(VolumeName : String) : String;

implementation

Uses
  JwaWinIoctl, IsAdmin, ShellApi, ShlObj, JwaWinBase,
  JwaNative, JwaNtStatus, JwaWintype, Math;

function GetVolumeMountPoints(VolumeName : String) : String;
Var
  Buf : ARRAY[0..1024] of Char;
  tmp,max : PChar;
  Written : DWORD;
  i : Integer;
begin
  Result:='';
  if GetVolumePathNamesForVolumeName(PChar(VolumeName),
    @Buf,sizeOf(Buf),Written) then begin
    tmp:=@Buf;
    max:=tmp;
    inc(max,Written);
    while ((tmp<max) and (tmp^ <> #0)) do begin
      i:=StrLen(tmp);
      If (i>0) Then begin
        If Length(Result)>0 Then result:=Result+',';
        Result:=Result+Strpas(tmp);
      end;
      inc(tmp,i+1);
    end;
  end;
end;

// PhysicalDiskName have to be the device name in form of \\.\PhysicalDdriveX
// Where X is the drive number
function GetVolumesOnPhysicalDisk(PhysicalDiskName : String; VolumeNames : TStrings) : Boolean;
Var
  tmp : String;
  i : Integer;
begin
  Result:=False;
  tmp:=AnsiLowerCase(PhysicalDiskName);
  i:=Pos('physicaldrive',tmp);
  if (i=0) then exit;
  Delete(tmp,1,i+12);
  i:=StrToIntDef(tmp,-1);
  if i<0 then exit;
  result:=GetVolumesOnPhysicalDisk(i,VolumeNames);
end;

function GetVolumesOnPhysicalDisk(PhysicalDiskNumber : Integer; VolumeNames : TStrings) : Boolean;
Var
  VolEnumHandle : THandle;
  Buf : ARRAY[0..1024] Of Char;
  VolumeName,tmp : String;
  i : Integer;
  Handle : THandle;
  Written : Cardinal;
begin
  VolEnumHandle:=FindFirstVolume(@Buf,sizeOf(Buf));
  if (VolEnumHandle <> INVALID_HANDLE_VALUE) then begin
    repeat
      VolumeName:=Strpas(@Buf);

      //Delete Backspace at the end, otherwise CreateFile will fail
      tmp:=VolumeName;
      i:=Length(tmp);
      If (tmp[i] = '\') Then
        Delete(tmp,i,1);

      Handle:=CreateFile(PChar(tmp),GENERIC_READ,
        FILE_SHARE_READ or FILE_SHARE_WRITE, nil, OPEN_EXISTING ,FILE_ATTRIBUTE_NORMAL,0);
      If (Handle <> INVALID_HANDLE_VALUE) then begin
        try
          If DeviceIoControl(Handle,IOCTL_VOLUME_GET_VOLUME_DISK_EXTENTS,nil,0,@Buf,sizeOf(Buf),@Written,nil) then
            with  PVOLUME_DISK_EXTENTS(@buf)^ do begin
              for i := 0 to NumberOfDiskExtents-1 do
                if (Extents[i].DiskNumber = PhysicalDiskNumber) Then
                  VolumeNames.Add(VolumeName);
            end;
        finally
          CloseHandle(Handle);
        end;
      end;
    until not FindNextVolume(VolEnumHandle,@Buf,sizeOf(Buf));
    FindVolumeClose(VolEnumHandle);
    Result:=True;
  end;
end;

const
  ByteMatrics : ARRAY[0..6] of String = ('Byte','KB','MB','GB','TB','PB','EB');

function FormatByteValue(bytes : Int64) : String;
Var
  I : Integer;
  tmp,Divisor : Int64;
  Value : Double;
begin
  tmp:=bytes;
  i:=0;
  repeat
    tmp := tmp shr 10;
    inc(i);
  until (tmp<1024);
  Divisor:=1;
  if (I>0) Then Divisor:=Divisor shl (I*10);
  Value:=Bytes / Divisor;
  Result:=Format('%2f %s',[Value,ByteMatrics[I]]);
end;


var
  PARTITION_TYPES : ARRAY[Byte] Of String;


function GetPartitionType(PartitionType : Byte; Default : String) : String;
begin
  Result := PARTITION_TYPES[PartitionType];
  If (Result='') Then Result := default;
end;

begin
  PARTITION_TYPES[$00] := 'Unused';
  PARTITION_TYPES[$01] := 'FAT 12';
  PARTITION_TYPES[$02] := 'Xenix 1';
  PARTITION_TYPES[$03] := 'Xenix 2';
  PARTITION_TYPES[$04] := 'FAT_16';
  PARTITION_TYPES[$05] := 'EXTENDED';
  PARTITION_TYPES[$06] := 'HUGE';
  PARTITION_TYPES[$07] := 'NTFS/IFS';
  PARTITION_TYPES[$0A] := 'OS2BOOTMGR';
  PARTITION_TYPES[$0B] := 'FAT32';
  PARTITION_TYPES[$0C] := 'FAT32_XINT13';
  PARTITION_TYPES[$0E] := 'XINT13';
  PARTITION_TYPES[$0F] := 'XINT13_EXTENDED';
  PARTITION_TYPES[$41] := 'PREP';
  PARTITION_TYPES[$42] := 'LDM';
  PARTITION_TYPES[$63] := 'UNIX';
  PARTITION_TYPES[$C0] := 'NTFT';
  PARTITION_TYPES[$82] := 'Linux Swap';
  PARTITION_TYPES[$83] := 'Linux';
end.
