unit MainForm;

{$DEFINE AdminCheck}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  StdCtrls, ComCtrls, StrUtils, WbemScripting_TLB, EraseThread,
  ExtCtrls, ActiveX, Dialogs;

type
  TForm1 = class(TForm)
    EraseDrive: TButton;
    DriveList: TComboBox;
    Label1: TLabel;
    DriveProperties: TListView;
    Timer1: TTimer;
    ProgressBar1: TProgressBar;
    ProgressBar2: TProgressBar;
    Label2: TLabel;
    Label3: TLabel;
    StatusBar1: TStatusBar;
    EraseMode: TComboBox;
    Label4: TLabel;
    BufferSize: TTrackBar;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    procedure FormShow(Sender: TObject);
    procedure DriveListChange(Sender: TObject);
    procedure EraseDriveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    DeviceID : String;
    BytesPerSector : DWord;
    EraseThread : TEraseThread;
    procedure RestoreCaption;
  public
    procedure AddWMIDriveProperty(WMIItem : ISWBemObject; PropertyName : String);
    procedure AddDriveProperty(PropertyName, PropertyValue : String);
    function IsSystemDrive(DeviceName : String) : Boolean;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

Uses
  JwaWinIoctl, DiskTools, IsAdmin, ShellApi, ShlObj, JwaWinBase,
  JwaNative, JwaNtStatus, JwaWintype;

const
  SNoAdmin = 'The current logged on account does not have administrative permissions'+#13+#10+
             'Without administrative permissions DeviceEraser will not work.';
  SEraseWarning = 'Do you really want to erase the Device "%s"?'+#13+
                  'All data on this drive will be lost!';
  SEraseLastWarning = #13+'This is the last warning!';
  SEraseAbortQuestion = 'Erase is running! Do you want to stop the current erase procedure?';

function GetPhysicalDrives() : String;
Var
  I : Integer;
  H : THandle;
  S : String;
begin
  Result:='';
  H:=0;
  For I:=0 TO 100 do begin
    Try
      // The try and error method for finding the installed physical disks
      S:=Format('\\.\PhysicalDrive%d',[I]);
      H:=CreateFile(PChar(S),GENERIC_READ, 0, nil, OPEN_EXISTING ,0,0);
      IF (H<> INVALID_HANDLE_VALUE) Then Result:=Result + S + #13+#10;
    Finally
      CloseHandle(H);
      H:=0;
    End;
  end;
end;

function ExecuteWMIQuery(Query : String) : IEnumVariant;
Var
  WMIServices : ISWbemServices;
  WMIQuerySet : ISWbemObjectSet;
begin
  WMIServices:=CoSWbemLocator.Create.ConnectServer('.', 'root\cimv2', '','', '', '', 0, nil);
  WMIQuerySet:=WMIServices.ExecQuery(Query, 'WQL', wbemFlagBidirectional, nil);
  Result:=WMIQuerySet._NewEnum as IEnumVariant;
end;

function GetNextWMIResult(Enum : IEnumVariant; Var WMIResult : ISWBemObject) : Boolean;
Var
  EnumItem : OleVariant;
  Count : LongWord;
begin
  Result:=Enum.Next(1,EnumItem,Count) = S_OK;
  If Result Then
    WMIResult:= (IUnknown(EnumItem) as SWBemObject)
  else
    WMIresult:=nil;
end;

function TForm1.IsSystemDrive(DeviceName : String) : Boolean;
var
  path : array[0..MAX_PATH] of Char;
begin
  Result:=false;
  If NOT SHGetSpecialFolderPath(0,path,$24,false) //Get Windows-Dir path
  Then RaiseLastOSError;
  Messagebox(Handle,Path,'debug',MB_OK);
  If NOT GetVolumePathName(path,path,sizeOf(Path)) Then
    RaiseLastOSError;
  Messagebox(Handle,Path,'debug',MB_OK);
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  {$IFDEF AdminCheck}
  if not IsCurrentUserLocalAdministrator() then
    MessageDlg(SNoAdmin,mtWarning, [mbOk], 0);
  {$ENDIF}
  ShowSimulationInfo(self);
  DriveList.Items.Text:=GetPhysicalDrives();
  EraseMode.ItemIndex:=0;
end;

procedure TForm1.DriveListChange(Sender: TObject);
const
  query : String = 'Select * from Win32_DiskDrive Where DeviceID=';
var
  EscapedDeviceID : String;
  Enum : IEnumVariant;
  WMIItem : ISWBemObject;
  size : Int64;
  H : THandle;
  i : Integer;
  discInfo : DISK_GEOMETRY_EX;
  pDriveLayout : PDRIVE_LAYOUT_INFORMATION_EX;
  buffer : ARRAY[0..4095] of Byte;
  bout : Cardinal;
  res : LongBool;
  s,t : String;
  Volumes : TStringList;
begin
  IF (DriveList.ItemIndex<0) Then exit;
  DeviceID:=DriveList.Items[DriveList.ItemIndex];
  BytesPerSector:=0;
  EscapedDeviceID:=AnsiReplaceStr(DeviceID,'\','\\');
  DriveProperties.Items.BeginUpdate;
  try
    DriveProperties.Items.Clear;
    AddDriveProperty('Name',DeviceID);
    try
      Enum := ExecuteWMIQuery(query + Chr(39) +EscapedDeviceID + Chr(39));
      while GetNextWMIResult(Enum,WMIItem) do begin
        //AddWMIDriveProperty(WMIItem,'Manufacturer');
        //AddWMIDriveProperty(WMIItem,'Name');
        AddWMIDriveProperty(WMIItem,'Model');
        AddWMIDriveProperty(WMIItem,'Description');
        AddWMIDriveProperty(WMIItem,'InterfaceType');
        AddWMIDriveProperty(WMIItem,'Partitions');
      end;
    except
    end;
    H:=CreateFile(PChar(DeviceID),GENERIC_READ, 0, nil, OPEN_EXISTING ,0,0);
    try
      BytesPerSector:=0;
      size:=0;
      res:=Windows.DeviceIOControl(H,IOCTL_DISK_GET_DRIVE_GEOMETRY_EX,nil,0,@discInfo,sizeOf(discInfo),bout,nil);
      if (res) then begin
        BytesPerSector:=discInfo.Geometry.BytesPerSector;
        size:=discInfo.DiskSize.QuadPart;
        AddDriveProperty('Disk Size',FormatByteValue(size));
      end;

      res:=Windows.DeviceIOControl(H,IOCTL_DISK_GET_DRIVE_LAYOUT_EX,nil,0,@buffer,sizeOf(buffer),bout,nil);
      if (res) then begin
        pDriveLayout:=@Buffer;
        with pDriveLayout^ do begin
          For i:=1 to PartitionCount do begin
            With PartitionEntry[i-1] do begin
              Case  PartitionStyle of
                PARTITION_STYLE_MBR :
                with Mbr do begin
                  If (PartitionType <> 0) Then
                  AddDriveProperty(Format('  Partition %d',[i]),
                    Format('Type: %s - Size: %s',[GetPartitionType(PartitionType,'?'),
                    FormatByteValue(PartitionLength.QuadPart)]));
                end;
                PARTITION_STYLE_GPT :
                with Gpt do begin
                  AddDriveProperty(Format('  Partition %d',[i]),Format('Dynamic volume "%s"',[Name]));
                end;
                PARTITION_STYLE_RAW :
                begin
                  AddDriveProperty(Format('  Partition %d',[i]),'Type: RAW/unknown');
                end;
              End;
            end;
          end;
        end;
      end;
    finally
      CloseHandle(H);
    end;
    Volumes := TStringList.Create;
    Volumes.Capacity:=10;
    if GetVolumesOnPhysicalDisk(DeviceID,Volumes) then begin
      s:='';
      for i:= 0 to Volumes.Count - 1 do begin
        //AddDriveProperty('Volume',Volumes.Strings[i]);
        t:=GetVolumeMountPoints(Volumes.Strings[i]);
        if (Length(t)=0) then continue;
        if (Length(s)>0) then s:=s + ',';
        s:=s+t;
      end;
      If length(s)>0 Then begin
        AddDriveProperty('Mounted volumes',s);
      end;
    end;
    Volumes.Free;

  finally
      DriveProperties.Items.EndUpdate;
  end;
end;

procedure TForm1.AddWMIDriveProperty(WMIItem : ISWBemObject; PropertyName : String);
Var
  PropValue : ISWbemProperty;
begin
  PropValue := WMIItem.Properties_.Item(PropertyName,0);
  AddDriveProperty(PropertyName,VarToStr(PropValue));
end;

procedure TForm1.AddDriveProperty(PropertyName, PropertyValue : String);
Var
  ListItem : TListItem;
begin
  ListItem := DriveProperties.Items.Add;
  ListItem.Caption:=PropertyName;
  ListItem.SubItems.Add(Propertyvalue);
end;

procedure TForm1.EraseDriveClick(Sender: TObject);
Var
  T : TThread;
  EraseMethod : TEraseMethod;
begin
  IF Assigned(EraseThread) AND EraseThread.Running Then begin
    If (MessageBox(handle,PChar(SEraseAbortQuestion),'Confirm abortion',
      MB_YESNO or MB_ICONQUESTION or MB_DEFBUTTON2) = IDNO) then exit;
    EraseThread.Terminate;
    EraseDrive.Caption:='Erase Drive';
  end else begin
    IF (DeviceID='') Then exit;
    If (MessageBox(handle,PChar(Format(SEraseWarning,[DeviceID])),
      'Confirm erasure', MB_YESNO or MB_ICONQUESTION or MB_DEFBUTTON2) = IDNO)
      then exit;
    If (MessageBox(handle,PChar(Format(SEraseWarning+SEraseLastWarning,[DeviceID])),
      'Confirm erasure', MB_YESNO or MB_ICONQUESTION or MB_DEFBUTTON2) = IDNO)
      then exit;
    IF Assigned(EraseThread) Then begin
      T:=EraseThread;
      EraseThread:=nil;
      T.Free;
    end;
    EraseMethod:=TEraseMethod(EraseMode.Items.Objects[EraseMode.ItemIndex]);
    EraseThread:=TEraseThread.Create(DeviceID,EraseMethod,BufferSize.Position);
    EraseDrive.Caption:='Cancel';
    Timer1.Enabled:=true;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  EraseThread:=nil;
  DeviceID:='';
  BytesPerSector:=0;

  EraseMode.Items.AddObject('Zeros     - 0x00',TEraseMethod.Create([ecZero]));
  EraseMode.Items.AddObject('Ones      - 0xFF',TEraseMethod.Create([ecOne]));
  EraseMode.Items.AddObject('Random - RND',TEraseMethod.Create([ecRandom]));
  EraseMode.Items.AddObject('3Times    - 0xFF RND 0x00',TEraseMethod.Create([ecOne, ecRandom, ecZero]));
  EraseMode.Items.AddObject('DOD      - 0x00 0xFF RND',TEraseMethod.Create([ecZero, ecOne, ecRandom]));
  RestoreCaption;
end;

procedure TForm1.RestoreCaption;
begin
  Caption:='DeviceEraser v1.0';
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  IF Assigned(EraseThread) Then EraseThread.Free;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
Var
  Left,Total,Done : Int64;
  progress : Single;
  CycleProgress : Single;
  S : String;
  i : integer;
begin
  IF NOT Assigned(EraseThread) Then begin
    Timer1.Enabled:=false;
    EraseDrive.Caption:='Erase Drive';
    exit;
  end;
  if (NOT EraseThread.Running) then begin
    EraseDrive.Caption:='Erase Drive';
    ProgressBar1.Position:=0;
    ProgressBar1.Repaint;
    ProgressBar2.Position:=0;
    ProgressBar2.Repaint;
    Timer1.Enabled:=false;
    RestoreCaption;
  end else begin
    Left:=EraseThread.BytesLeft;
    Total:=EraseThread.TotalBytes;
    Done:=Total-Left;
    s:=FormatByteValue(Done);
    progress := 100 * Done;
    progress := progress / Total;
    CycleProgress := 100 * (EraseThread.CurrentCycle-1) / EraseThread.CyclesCount;
    ProgressBar1.Position:=Round(progress);
    ProgressBar1.Repaint;

    ProgressBar2.Position:=Round(CycleProgress + (progress / EraseThread.CyclesCount));
    ProgressBar2.Repaint;
    Caption:=Format('Progress: %s = %2f%% (cycle %d of %d)',[s,progress,EraseThread.CurrentCycle,EraseThread.CyclesCount]);
  end;
end;

end.

